/**
 * @file
 * Grid implementation.
 *
 * @copyright Copyright © 2017–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                     "title" : "Time Curve",
					 "keywords" : [ ],
                     "version" : "2.0.0",
                     "genericTypes" : {
                          "VuoGenericType1" : {
                              "compatibleTypes" :["VuoReal", "VuoPoint2d", "VuoPoint3d" ]
                          }
                      },
					 "node": {
                          "exampleCompositions" : [ "RepeatingTimer.vuo" ]
					 }
				 });

void nodeEvent
(
        VuoInputData(VuoReal) time,
        VuoInputData(VuoReal) startTime,
        VuoInputData(VuoReal) endTime,
        VuoInputData(VuoGenericType1) startPosition,
        VuoInputData(VuoGenericType1) endPosition,
        VuoInputData(VuoCurve,{"default" : "quadratic"}) curve,
        VuoInputData(VuoCurveEasing, {"default" : "in"}) easing,
        VuoOutputData(VuoGenericType1) value
)
{
    //Calculate timings
    VuoReal runTime = time - startTime;         //Sets global time to negative for the node until the startTime is met
    VuoReal duration = endTime - startTime;     //Calculates the duration based on start and end times
    VuoReal currentTime = 0.0;                  //Creates a relative timer float for the node

    //Calculate only if needed

    if (runTime < 0.0) {
        currentTime = 0.0;
    } else if (runTime > duration) {
        currentTime = duration;
    } else {
        currentTime = runTime;
    }

    *value = VuoGenericType1_curve(             //Create the curve at the item time
                currentTime,
                startPosition,
                endPosition,
                duration,
                curve,
                easing,
                VuoLoopType_None
                );
}
