/**
 * @file
 * Grid implementation.
 *
 * @copyright Copyright © 2017–2018 Magneson.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

VuoModuleMetadata({
                     "title" : "Time Curve List",
					 "keywords" : [ ],
                     "version" : "2.0.0",
                     "genericTypes" : {
                          "VuoGenericType1" : {
                              "compatibleTypes" : ["VuoReal", "VuoPoint2d", "VuoPoint3d"]
                          }
                      },
					 "node": {
                          "exampleCompositions" : [ "MultiStepCircles.vuo" ]
					 }
				 });

void nodeEvent
(
        VuoInputData(VuoReal) time,
        VuoInputData(VuoList_VuoReal) positionTimes,
        VuoInputData(VuoList_VuoGenericType1) positions,
        VuoInputData(VuoList_VuoCurve,{"default" : "quadratic"}) curve,
        VuoInputData(VuoList_VuoCurveEasing, {"default" : "in"}) easing,
        VuoOutputData(VuoGenericType1) value
)
{
    VuoInteger numElements = VuoListGetCount_VuoReal(positionTimes);                                                    //Get the amount of time points

    for (int i = 0; i < numElements; i++){
        VuoReal currentTime = time - VuoListGetValue_VuoReal(positionTimes, i);
        VuoReal duration = VuoListGetValue_VuoReal(positionTimes, i + 1) - VuoListGetValue_VuoReal(positionTimes, i);

        if(VuoListGetValue_VuoReal(positionTimes, numElements) < time) {
            break;                                                                                                      //Break the loop if no more events exist after the current time (not sure if this works)
        } else if(currentTime > 0.0 && currentTime < (currentTime + duration)) {                                        //If time is within an event, calculate the curve

        VuoGenericType1 startPosition = VuoListGetValue_VuoGenericType1(positions, i);                                  //Where the point should be
        VuoGenericType1 endPosition = VuoListGetValue_VuoGenericType1(positions, i + 1);                                //Where the point is supposed to go
        VuoCurve itemCurve = VuoListGetValue_VuoCurve(curve, i);                                                        //Get the curve for the transition
        VuoCurveEasing itemEasing = VuoListGetValue_VuoCurveEasing(easing, i);                                          //Get the easing for the transition

        *value = VuoGenericType1_curve(
                    currentTime,
                    startPosition,
                    endPosition,
                    duration,
                    itemCurve,
                    itemEasing,
                    VuoLoopType_None                                                                                    //No loops wanted
                    );

        }
    }
}
